library angular2.src.core.compiler.view_container;

import "package:angular2/src/facade/collection.dart"
    show ListWrapper, MapWrapper, List;
import "package:angular2/src/facade/lang.dart" show BaseException;
import "package:angular2/di.dart" show Injector;
import "package:angular2/src/core/compiler/element_injector.dart" as eiModule;
import "package:angular2/src/facade/lang.dart" show isPresent, isBlank;
import "view.dart" as viewModule;
import "package:angular2/src/render/api.dart"
    show ViewContainerRef; /**
 * @exportedAs angular2/view
 */

class ViewContainer {
  viewModule.AppView parentView;
  viewModule.AppProtoView defaultProtoView;
  List<viewModule.AppView> _views;
  eiModule.ElementInjector elementInjector;
  ViewContainer(viewModule.AppView parentView,
      viewModule.AppProtoView defaultProtoView,
      eiModule.ElementInjector elementInjector) {
    this.parentView = parentView;
    this.defaultProtoView = defaultProtoView;
    this.elementInjector =
        elementInjector; // The order in this list matches the DOM order.
    this._views = [];
  }
  getRender() {
    return new ViewContainerRef(
        this.parentView.render, this.elementInjector.getBoundElementIndex());
  }
  internalClearWithoutRender() {
    for (var i = this._views.length - 1; i >= 0; i--) {
      this._detachInjectors(i);
    }
  }
  clear() {
    for (var i = this._views.length - 1; i >= 0; i--) {
      this.remove(i);
    }
  }
  viewModule.AppView get(num index) {
    return this._views[index];
  }
  get length {
    return this._views.length;
  }
  _siblingInjectorToLinkAfter(num index) {
    if (index == 0) return null;
    return ListWrapper.last(this._views[index - 1].rootElementInjectors);
  }
  hydrated() {
    return this.parentView.hydrated();
  } // TODO(rado): profile and decide whether bounds checks should be added
  // to the methods below.
  viewModule.AppView create([atIndex = -1,
      viewModule.AppProtoView protoView = null, Injector injector = null]) {
    if (atIndex == -1) atIndex = this._views.length;
    if (!this.hydrated()) throw new BaseException(
        "Cannot create views on a dehydrated ViewContainer");
    if (isBlank(protoView)) {
      protoView = this.defaultProtoView;
    }
    var newView = this.parentView.viewFactory.getView(
        protoView); // insertion must come before hydration so that element injector trees are attached.
    this._insertInjectors(newView, atIndex);
    this.parentView.viewHydrator.hydrateViewInViewContainer(
        this, atIndex, newView, injector);
    return newView;
  }
  viewModule.AppView insert(view, [atIndex = -1]) {
    if (atIndex == -1) atIndex = this._views.length;
    this._insertInjectors(view, atIndex);
    this.parentView.changeDetector.addChild(view.changeDetector);
    this.parentView.renderer.insertViewIntoContainer(
        this.getRender(), atIndex, view.render);
    return view;
  }
  viewModule.AppView _insertInjectors(view, atIndex) {
    ListWrapper.insert(this._views, atIndex, view);
    this._linkElementInjectors(this._siblingInjectorToLinkAfter(atIndex), view);
    return view;
  }
  indexOf(viewModule.AppView view) {
    return ListWrapper.indexOf(this._views, view);
  }
  remove([atIndex = -1]) {
    if (atIndex == -1) atIndex = this._views.length - 1;
    var view = this._views[atIndex]; // opposite order as in create
    this.parentView.viewHydrator.dehydrateViewInViewContainer(
        this, atIndex, view);
    this._detachInjectors(atIndex);
    this.parentView.viewFactory.returnView(view);
  } /**
   * The method can be used together with insert to implement a view move, i.e.
   * moving the dom nodes while the directives in the view stay intact.
   */
  viewModule.AppView detach([atIndex = -1]) {
    if (atIndex == -1) atIndex = this._views.length - 1;
    var detachedView = this._detachInjectors(atIndex);
    detachedView.changeDetector.remove();
    this.parentView.renderer.detachViewFromContainer(this.getRender(), atIndex);
    return detachedView;
  }
  viewModule.AppView _detachInjectors(atIndex) {
    var detachedView = this.get(atIndex);
    ListWrapper.removeAt(this._views, atIndex);
    this._unlinkElementInjectors(detachedView);
    return detachedView;
  }
  _linkElementInjectors(sibling, view) {
    for (var i = view.rootElementInjectors.length - 1; i >= 0; i--) {
      view.rootElementInjectors[i].linkAfter(this.elementInjector, sibling);
    }
  }
  _unlinkElementInjectors(view) {
    for (var i = 0; i < view.rootElementInjectors.length; ++i) {
      view.rootElementInjectors[i].unlink();
    }
  }
}
