library angular2.src.core.compiler.view_factory;

import "package:angular2/di.dart" show Injectable, Inject, OpaqueToken;
import "package:angular2/src/facade/collection.dart"
    show ListWrapper, MapWrapper, Map, StringMapWrapper, List;
import "element_injector.dart" as eli;
import "package:angular2/src/facade/lang.dart"
    show isPresent, isBlank, BaseException;
import "package:angular2/src/core/compiler/ng_element.dart" show NgElement;
import "view.dart" as viewModule;
import "package:angular2/src/render/api.dart"
    show
        Renderer; // TODO(tbosch): Make this an OpaqueToken as soon as our transpiler supports this!

const VIEW_POOL_CAPACITY = "ViewFactory.viewPoolCapacity";
@Injectable()
class ViewFactory {
  num _poolCapacityPerProtoView;
  Map<viewModule.AppProtoView, List<viewModule.AppView>> _pooledViewsPerProtoView;
  Renderer _renderer;
  ViewFactory(
      @Inject(VIEW_POOL_CAPACITY) poolCapacityPerProtoView, Renderer renderer) {
    this._poolCapacityPerProtoView = poolCapacityPerProtoView;
    this._pooledViewsPerProtoView = MapWrapper.create();
    this._renderer = renderer;
  }
  viewModule.AppView getView(viewModule.AppProtoView protoView) {
    var pooledViews = MapWrapper.get(this._pooledViewsPerProtoView, protoView);
    if (isPresent(pooledViews) && pooledViews.length > 0) {
      return ListWrapper.removeLast(pooledViews);
    }
    return this._createView(protoView);
  }
  returnView(viewModule.AppView view) {
    if (view.hydrated()) {
      throw new BaseException(
          "Only dehydrated Views can be put back into the pool!");
    }
    var protoView = view.proto;
    var pooledViews = MapWrapper.get(this._pooledViewsPerProtoView, protoView);
    if (isBlank(pooledViews)) {
      pooledViews = [];
      MapWrapper.set(this._pooledViewsPerProtoView, protoView, pooledViews);
    }
    if (pooledViews.length < this._poolCapacityPerProtoView) {
      ListWrapper.push(pooledViews, view);
    }
  }
  viewModule.AppView _createView(viewModule.AppProtoView protoView) {
    var view = new viewModule.AppView(
        this._renderer, this, protoView, protoView.protoLocals);
    var changeDetector = protoView.protoChangeDetector.instantiate(view,
        protoView.bindings, protoView.getVariableBindings(),
        protoView.getdirectiveRecords());
    var binders = protoView.elementBinders;
    var elementInjectors = ListWrapper.createFixedSize(binders.length);
    var rootElementInjectors = [];
    var preBuiltObjects = ListWrapper.createFixedSize(binders.length);
    var componentChildViews = ListWrapper.createFixedSize(binders.length);
    for (var binderIdx = 0; binderIdx < binders.length; binderIdx++) {
      var binder = binders[binderIdx];
      var elementInjector = null; // elementInjectors and rootElementInjectors
      var protoElementInjector = binder.protoElementInjector;
      if (isPresent(protoElementInjector)) {
        if (isPresent(protoElementInjector.parent)) {
          var parentElementInjector = elementInjectors[
              protoElementInjector.parent.index];
          elementInjector =
              protoElementInjector.instantiate(parentElementInjector);
        } else {
          elementInjector = protoElementInjector.instantiate(null);
          ListWrapper.push(rootElementInjectors, elementInjector);
        }
      }
      elementInjectors[binderIdx] = elementInjector; // componentChildViews
      var childChangeDetector = null;
      if (binder.hasStaticComponent()) {
        var childView = this._createView(binder.nestedProtoView);
        childChangeDetector = childView.changeDetector;
        changeDetector.addShadowDomChild(childChangeDetector);
        componentChildViews[binderIdx] = childView;
      } // preBuiltObjects
      if (isPresent(elementInjector)) {
        preBuiltObjects[binderIdx] = new eli.PreBuiltObjects(
            view, new NgElement(view, binderIdx), childChangeDetector);
      }
    }
    view.init(changeDetector, elementInjectors, rootElementInjectors,
        preBuiltObjects, componentChildViews);
    return view;
  }
}
