library angular2.test.core.compiler.view_factory_spec;

import "package:angular2/test_lib.dart"
    show
        AsyncTestCompleter,
        beforeEach,
        ddescribe,
        xdescribe,
        describe,
        el,
        dispatchEvent,
        expect,
        iit,
        inject,
        beforeEachBindings,
        it,
        xit,
        SpyObject,
        proxy;
import "package:angular2/src/facade/lang.dart" show IMPLEMENTS, isBlank;
import "package:angular2/src/core/compiler/view_factory.dart" show ViewFactory;
import "package:angular2/src/render/api.dart" show Renderer, ViewRef;
import "package:angular2/src/core/compiler/view.dart"
    show AppProtoView, AppView;
import "package:angular2/src/core/compiler/element_injector.dart"
    show DirectiveBinding, ElementInjector;
import "package:angular2/src/core/compiler/directive_metadata_reader.dart"
    show DirectiveMetadataReader;
import "package:angular2/src/core/annotations/annotations.dart" show Component;
import "package:angular2/src/core/compiler/element_binder.dart"
    show ElementBinder;
import "package:angular2/change_detection.dart"
    show ChangeDetector, ProtoChangeDetector;

main() {
  describe("AppViewFactory", () {
    var reader;
    var renderer;
    beforeEach(() {
      renderer = new SpyRenderer();
      reader = new DirectiveMetadataReader();
    });
    ViewFactory createViewFactory({capacity}) {
      return new ViewFactory(capacity, renderer);
    }
    createProtoChangeDetector() {
      var pcd = new SpyProtoChangeDetector();
      pcd.spy("instantiate").andCallFake(
          (dispatcher, bindingRecords, variableBindings, directiveRecords) {
        return new SpyChangeDetector();
      });
      return pcd;
    }
    createProtoView([binders = null]) {
      if (isBlank(binders)) {
        binders = [];
      }
      var pv = new AppProtoView(null, createProtoChangeDetector());
      pv.elementBinders = binders;
      return pv;
    }
    createDirectiveBinding(type) {
      var meta = reader.read(type);
      return DirectiveBinding.createFromType(meta.type, meta.annotation);
    }
    createComponentElBinder(binding, [nestedProtoView = null]) {
      var binder = new ElementBinder(0, null, 0, null, binding, null);
      binder.nestedProtoView = nestedProtoView;
      return binder;
    }
    it("should create views without cache", () {
      var pv = createProtoView();
      var vf = createViewFactory(capacity: 0);
      expect(vf.getView(pv) is AppView).toBe(true);
    });
    describe("caching", () {
      it("should support multiple AppProtoViews", () {
        var pv1 = createProtoView();
        var pv2 = createProtoView();
        var vf = createViewFactory(capacity: 2);
        var view1 = vf.getView(pv1);
        var view2 = vf.getView(pv2);
        vf.returnView(view1);
        vf.returnView(view2);
        expect(vf.getView(pv1)).toBe(view1);
        expect(vf.getView(pv2)).toBe(view2);
      });
      it("should reuse the newest view that has been returned", () {
        var pv = createProtoView();
        var vf = createViewFactory(capacity: 2);
        var view1 = vf.getView(pv);
        var view2 = vf.getView(pv);
        vf.returnView(view1);
        vf.returnView(view2);
        expect(vf.getView(pv)).toBe(view2);
      });
      it("should not add views when the capacity has been reached", () {
        var pv = createProtoView();
        var vf = createViewFactory(capacity: 2);
        var view1 = vf.getView(pv);
        var view2 = vf.getView(pv);
        var view3 = vf.getView(pv);
        vf.returnView(view1);
        vf.returnView(view2);
        vf.returnView(view3);
        expect(vf.getView(pv)).toBe(view2);
        expect(vf.getView(pv)).toBe(view1);
      });
    });
    describe("child components", () {
      var vf;
      beforeEach(() {
        vf = createViewFactory(capacity: 1);
      });
      it("should create static child component views", () {
        var hostPv = createProtoView([
          createComponentElBinder(
              createDirectiveBinding(SomeComponent), createProtoView())
        ]);
        var hostView = vf.getView(hostPv);
        var shadowView = hostView.componentChildViews[0];
        expect(shadowView).toBeTruthy();
        expect(hostView.changeDetector.spy("addShadowDomChild"))
            .toHaveBeenCalledWith(shadowView.changeDetector);
      });
      it("should not create dynamic child component views", () {
        var hostPv = createProtoView([
          createComponentElBinder(createDirectiveBinding(SomeComponent), null)
        ]);
        var hostView = vf.getView(hostPv);
        var shadowView = hostView.componentChildViews[0];
        expect(shadowView).toBeFalsy();
      });
    });
  });
}
@Component(selector: "someComponent")
class SomeComponent {}
@proxy
@IMPLEMENTS(Renderer)
class SpyRenderer extends SpyObject implements Renderer {
  SpyRenderer() : super(Renderer) {
    /* super call moved to initializer */;
  }
  noSuchMethod(m) {
    return super.noSuchMethod(m);
  }
}
@proxy
@IMPLEMENTS(ChangeDetector)
class SpyChangeDetector extends SpyObject implements ChangeDetector {
  SpyChangeDetector() : super(ChangeDetector) {
    /* super call moved to initializer */;
  }
  noSuchMethod(m) {
    return super.noSuchMethod(m);
  }
}
@proxy
@IMPLEMENTS(ProtoChangeDetector)
class SpyProtoChangeDetector extends SpyObject implements ProtoChangeDetector {
  SpyProtoChangeDetector() : super(ProtoChangeDetector) {
    /* super call moved to initializer */;
  }
  noSuchMethod(m) {
    return super.noSuchMethod(m);
  }
}
