library angular2.test.core.compiler.view_hydrator_spec;

import "package:angular2/test_lib.dart"
    show
        AsyncTestCompleter,
        beforeEach,
        ddescribe,
        xdescribe,
        describe,
        el,
        dispatchEvent,
        expect,
        iit,
        inject,
        beforeEachBindings,
        it,
        xit,
        SpyObject,
        proxy;
import "package:angular2/src/facade/lang.dart"
    show IMPLEMENTS, isBlank, isPresent;
import "package:angular2/src/facade/collection.dart"
    show MapWrapper, ListWrapper, StringMapWrapper;
import "package:angular2/src/core/compiler/view.dart"
    show AppProtoView, AppView;
import "package:angular2/src/render/api.dart" show Renderer, ViewRef;
import "package:angular2/change_detection.dart" show ChangeDetector;
import "package:angular2/src/core/compiler/element_binder.dart"
    show ElementBinder;
import "package:angular2/src/core/compiler/element_injector.dart"
    show DirectiveBinding, ElementInjector, ElementRef;
import "package:angular2/src/core/compiler/directive_metadata_reader.dart"
    show DirectiveMetadataReader;
import "package:angular2/src/core/annotations/annotations.dart" show Component;
import "package:angular2/src/core/compiler/view_hydrator.dart"
    show AppViewHydrator;
import "package:angular2/src/core/compiler/view_factory.dart" show ViewFactory;

main() {
  describe("AppViewHydrator", () {
    var renderer;
    var reader;
    var hydrator;
    var viewFactory;
    beforeEach(() {
      renderer = new SpyRenderer();
      reader = new DirectiveMetadataReader();
      viewFactory = new SpyViewFactory();
      hydrator = new AppViewHydrator(renderer, viewFactory);
    });
    createDirectiveBinding(type) {
      var meta = reader.read(type);
      return DirectiveBinding.createFromType(meta.type, meta.annotation);
    }
    createElementInjector(overrides) {
      return SpyObject.stub(new SpyElementInjector(), {
        "isExportingComponent": false,
        "isExportingElement": false,
        "getEventEmitterAccessors": [],
        "getComponent": null
      }, overrides);
    }
    createEmptyElBinder() {
      return new ElementBinder(0, null, 0, null, null, null);
    }
    createComponentElBinder(binding, [nestedProtoView = null]) {
      var binder = new ElementBinder(0, null, 0, null, binding, null);
      binder.nestedProtoView = nestedProtoView;
      return binder;
    }
    createProtoView([binders = null]) {
      if (isBlank(binders)) {
        binders = [];
      }
      var res = new AppProtoView(null, null);
      res.elementBinders = binders;
      return res;
    }
    createHostProtoView(nestedProtoView) {
      return createProtoView([
        createComponentElBinder(
            createDirectiveBinding(SomeComponent), nestedProtoView)
      ]);
    }
    createEmptyView() {
      var view =
          new AppView(renderer, null, createProtoView(), MapWrapper.create());
      var changeDetector = new SpyChangeDetector();
      view.init(changeDetector, [], [], [], []);
      return view;
    }
    createHostView(pv, shadowView, componentInstance,
        [elementInjectors = null]) {
      var view = new AppView(renderer, null, pv, MapWrapper.create());
      var changeDetector = new SpyChangeDetector();
      var eis;
      if (isPresent(elementInjectors)) {
        eis = elementInjectors;
      } else {
        eis = [createElementInjector({"getComponent": componentInstance})];
      }
      view.init(changeDetector, eis, eis,
          ListWrapper.createFixedSize(eis.length), [shadowView]);
      return view;
    }
    hydrate(view) {
      hydrator.hydrateInPlaceHostView(null, null, view, null);
    }
    dehydrate(view) {
      hydrator.dehydrateInPlaceHostView(null, view);
    }
    describe("hydrateDynamicComponentView", () {
      it("should not allow to use non component indices", () {
        var pv = createProtoView([createEmptyElBinder()]);
        var view = createHostView(pv, null, null);
        var shadowView = createEmptyView();
        expect(() => hydrator.hydrateDynamicComponentView(
                new ElementRef(null, view, 0, null), shadowView, null, null))
            .toThrowError(
                "There is no dynamic component directive at element 0");
      });
      it("should not allow to use static component indices", () {
        var pv = createHostProtoView(createProtoView());
        var view = createHostView(pv, null, null);
        var shadowView = createEmptyView();
        expect(() => hydrator.hydrateDynamicComponentView(
                new ElementRef(null, view, 0, null), shadowView, null, null))
            .toThrowError(
                "There is no dynamic component directive at element 0");
      });
      it("should not allow to overwrite an existing component", () {
        var pv = createHostProtoView(null);
        var shadowView = createEmptyView();
        var view = createHostView(pv, null, null);
        renderer
            .spy("createDynamicComponentView")
            .andReturn([new ViewRef(), new ViewRef()]);
        var elRef = new ElementRef(null, view, 0, null);
        hydrator.hydrateDynamicComponentView(
            elRef, shadowView, createDirectiveBinding(SomeComponent), null);
        expect(() => hydrator.hydrateDynamicComponentView(
                elRef, shadowView, null, null))
            .toThrowError("There already is a bound component at element 0");
      });
    });
    describe("hydrate... shared functionality", () {
      it("should hydrate existing child components", () {
        var hostPv = createHostProtoView(createProtoView());
        var componentInstance = new Object();
        var shadowView = createEmptyView();
        var hostView = createHostView(hostPv, shadowView, componentInstance);
        renderer.spy("createInPlaceHostView").andCallFake((a, b, c) {
          return [new ViewRef(), new ViewRef()];
        });
        hydrate(hostView);
        expect(shadowView.hydrated()).toBe(true);
      });
      it("should set up event listeners", () {
        var dir = new Object();
        var hostPv = createProtoView([
          createComponentElBinder(createDirectiveBinding(SomeComponent)),
          createEmptyElBinder()
        ]);
        var spyEventAccessor1 = SpyObject.stub({"subscribe": null});
        var ei1 = createElementInjector({
          "getEventEmitterAccessors": [[spyEventAccessor1]],
          "getDirectiveAtIndex": dir
        });
        var spyEventAccessor2 = SpyObject.stub({"subscribe": null});
        var ei2 = createElementInjector({
          "getEventEmitterAccessors": [[spyEventAccessor2]],
          "getDirectiveAtIndex": dir
        });
        var shadowView = createEmptyView();
        var hostView = createHostView(hostPv, shadowView, null, [ei1, ei2]);
        renderer
            .spy("createInPlaceHostView")
            .andReturn([new ViewRef(), new ViewRef()]);
        hydrate(hostView);
        expect(spyEventAccessor1.spy("subscribe")).toHaveBeenCalledWith(
            hostView, 0, dir);
        expect(spyEventAccessor2.spy("subscribe")).toHaveBeenCalledWith(
            hostView, 1, dir);
      });
    });
    describe("dehydrate... shared functionality", () {
      var hostView;
      var shadowView;
      createAndHydrate(nestedProtoView) {
        var componentInstance = new Object();
        shadowView = createEmptyView();
        var hostPv = createHostProtoView(nestedProtoView);
        hostView = createHostView(hostPv, shadowView, componentInstance);
        renderer
            .spy("createInPlaceHostView")
            .andReturn([new ViewRef(), new ViewRef()]);
        hydrate(hostView);
      }
      it("should dehydrate child components", () {
        createAndHydrate(createProtoView());
        dehydrate(hostView);
        expect(shadowView.hydrated()).toBe(false);
      });
      it("should not clear static child components", () {
        createAndHydrate(createProtoView());
        dehydrate(hostView);
        expect(hostView.componentChildViews[0]).toBe(shadowView);
        expect(hostView.changeDetector.spy("removeShadowDomChild")).not
            .toHaveBeenCalled();
        expect(viewFactory.spy("returnView")).not.toHaveBeenCalled();
      });
      it("should clear dynamic child components", () {
        createAndHydrate(null);
        dehydrate(hostView);
        expect(hostView.componentChildViews[0]).toBe(null);
        expect(hostView.changeDetector.spy("removeShadowDomChild"))
            .toHaveBeenCalledWith(shadowView.changeDetector);
        expect(viewFactory.spy("returnView")).toHaveBeenCalledWith(shadowView);
      });
      it("should clear imperatively added child components", () {
        createAndHydrate(createProtoView());
        var impHostView = createHostView(
            createHostProtoView(createProtoView()), createEmptyView(), null);
        shadowView.imperativeHostViews = [impHostView];
        dehydrate(hostView);
        expect(shadowView.imperativeHostViews).toEqual([]);
        expect(viewFactory.spy("returnView")).toHaveBeenCalledWith(impHostView);
        expect(shadowView.changeDetector.spy("removeChild"))
            .toHaveBeenCalledWith(impHostView.changeDetector);
      });
    });
  });
}
@Component(selector: "someComponent")
class SomeComponent {}
@proxy
@IMPLEMENTS(Renderer)
class SpyRenderer extends SpyObject implements Renderer {
  SpyRenderer() : super(Renderer) {
    /* super call moved to initializer */;
  }
  noSuchMethod(m) {
    return super.noSuchMethod(m);
  }
}
@proxy
@IMPLEMENTS(ChangeDetector)
class SpyChangeDetector extends SpyObject implements ChangeDetector {
  SpyChangeDetector() : super(ChangeDetector) {
    /* super call moved to initializer */;
  }
  noSuchMethod(m) {
    return super.noSuchMethod(m);
  }
}
@proxy
@IMPLEMENTS(ElementInjector)
class SpyElementInjector extends SpyObject implements ElementInjector {
  SpyElementInjector() : super(ElementInjector) {
    /* super call moved to initializer */;
  }
  noSuchMethod(m) {
    return super.noSuchMethod(m);
  }
}
@proxy
@IMPLEMENTS(ViewFactory)
class SpyViewFactory extends SpyObject implements ViewFactory {
  SpyViewFactory() : super(ViewFactory) {
    /* super call moved to initializer */;
  }
  noSuchMethod(m) {
    return super.noSuchMethod(m);
  }
}
