library angular2.test.core.compiler.view_spec;

import "package:angular2/test_lib.dart"
    show
        AsyncTestCompleter,
        beforeEach,
        ddescribe,
        xdescribe,
        describe,
        el,
        dispatchEvent,
        expect,
        iit,
        inject,
        beforeEachBindings,
        it,
        xit,
        SpyObject,
        proxy;
import "package:angular2/src/facade/lang.dart" show IMPLEMENTS, isBlank;
import "package:angular2/src/facade/collection.dart"
    show MapWrapper, ListWrapper;
import "package:angular2/src/core/compiler/view.dart"
    show AppProtoView, AppView;
import "package:angular2/src/core/compiler/view_container.dart"
    show ViewContainer;
import "package:angular2/src/render/api.dart" show Renderer;
import "package:angular2/change_detection.dart" show ChangeDetector;
import "package:angular2/src/core/compiler/element_binder.dart"
    show ElementBinder;
import "package:angular2/src/core/compiler/element_injector.dart"
    show ElementInjector;

main() {
  describe("AppView", () {
    var renderer;
    beforeEach(() {
      renderer = new SpyRenderer();
    });
    createElementInjector() {
      return new SpyElementInjector();
    }
    createEmptyElBinder() {
      return new ElementBinder(0, null, 0, null, null, null);
    }
    createEmbeddedProtoViewElBinder(nestedProtoView) {
      var binder = new ElementBinder(0, null, 0, null, null, null);
      binder.nestedProtoView = nestedProtoView;
      return binder;
    }
    createProtoView([binders = null]) {
      if (isBlank(binders)) {
        binders = [];
      }
      var res = new AppProtoView(null, null);
      res.elementBinders = binders;
      return res;
    }
    createViewWithOneBoundElement(pv) {
      var view = new AppView(renderer, null, pv, MapWrapper.create());
      var changeDetector = new SpyChangeDetector();
      var eij = createElementInjector();
      view.init(changeDetector, [eij], [eij], [null], [null]);
      return view;
    }
    describe("getOrCreateViewContainer()", () {
      it("should create a new container", () {
        var pv = createProtoView([createEmptyElBinder()]);
        var view = createViewWithOneBoundElement(pv);
        expect(view.getOrCreateViewContainer(0) is ViewContainer).toBe(true);
      });
      it("should return an existing container", () {
        var pv = createProtoView([createEmptyElBinder()]);
        var view = createViewWithOneBoundElement(pv);
        var vc = view.getOrCreateViewContainer(0);
        expect(view.getOrCreateViewContainer(0)).toBe(vc);
      });
      it("should store an existing nestedProtoView in the container", () {
        var defaultProtoView = createProtoView();
        var pv = createProtoView(
            [createEmbeddedProtoViewElBinder(defaultProtoView)]);
        var view = createViewWithOneBoundElement(pv);
        var vc = view.getOrCreateViewContainer(0);
        expect(vc.defaultProtoView).toBe(defaultProtoView);
      });
    });
  });
}
@proxy
@IMPLEMENTS(Renderer)
class SpyRenderer extends SpyObject implements Renderer {
  SpyRenderer() : super(Renderer) {
    /* super call moved to initializer */;
  }
  noSuchMethod(m) {
    return super.noSuchMethod(m);
  }
}
@proxy
@IMPLEMENTS(ChangeDetector)
class SpyChangeDetector extends SpyObject implements ChangeDetector {
  SpyChangeDetector() : super(ChangeDetector) {
    /* super call moved to initializer */;
  }
  noSuchMethod(m) {
    return super.noSuchMethod(m);
  }
}
@proxy
@IMPLEMENTS(ElementInjector)
class SpyElementInjector extends SpyObject implements ElementInjector {
  SpyElementInjector() : super(ElementInjector) {
    /* super call moved to initializer */;
  }
  noSuchMethod(m) {
    return super.noSuchMethod(m);
  }
}
